LOCAL_PATH:= $(call my-dir)

#-------------------------------------------------------------------------------
# A2DP plugin

include $(CLEAR_VARS)

LOCAL_SRC_FILES:= \
	a2dp.c \
	avdtp.c \
	control.c \
	device.c \
	gateway.c \
	headset.c \
	ipc.c \
	main.c \
	manager.c \
	media.c \
	module-bluetooth-sink.c \
	sink.c \
	source.c \
	telephony-dummy.c \
	transport.c \
	unix.c

LOCAL_CFLAGS:= \
	-DVERSION=\"4.93\" \
	-DSTORAGEDIR=\"/data/misc/bluetoothd\" \
	-DCONFIGDIR=\"/etc/bluetooth\" \
	-DANDROID \
	-D__S_IFREG=0100000  # missing from bionic stat.h

LOCAL_C_INCLUDES:= \
	$(LOCAL_PATH)/../lib \
	$(LOCAL_PATH)/../gdbus \
	$(LOCAL_PATH)/../src \
	$(LOCAL_PATH)/../btio \
	$(call include-path-for, glib) \
	$(call include-path-for, dbus)

LOCAL_SHARED_LIBRARIES := \
	libbluetooth \
	libbluetoothd \
	libbtio \
	libdbus \
	libglib


LOCAL_MODULE_PATH := $(TARGET_OUT_SHARED_LIBRARIES)/bluez-plugin
LOCAL_UNSTRIPPED_PATH := $(TARGET_OUT_SHARED_LIBRARIES_UNSTRIPPED)/bluez-plugin
LOCAL_MODULE := audio

include $(BUILD_SHARED_LIBRARY)

#-------------------------------------------------------------------------------
#
# liba2dp
# This is linked to Audioflinger so **LGPL only**

include $(CLEAR_VARS)

ifeq ($(findstring icx, $(TARGET_PRODUCT)),)
LOCAL_SRC_FILES:= \
	android_audio_hw.c \
	liba2dp.c \
	ipc.c \
	../sbc/sbc_primitives.c \
	../sbc/sbc_primitives_neon.c
else
LOCAL_SRC_FILES:= \
	lib/misc.c \
	lib/mute.c \
	lib/timed_mute.c \
	android_audio_hw_icx_src.c \
	android_audio_hw_icx_mixer.c \
	android_audio_hw_icx_buffer.c \
	android_audio_hw_icx_stream.c \
	android_audio_hw_icx.c \
	liba2dp.c \
	ipc.c \
	../sbc/sbc_primitives.c \
	../sbc/sbc_primitives_neon.c
endif

ifeq ($(TARGET_ARCH),x86)
LOCAL_SRC_FILES+= \
	../sbc/sbc_primitives_mmx.c \
	../sbc/sbc.c
else
LOCAL_SRC_FILES+= \
	../sbc/sbc.c.arm \
	../sbc/sbc_primitives_armv6.c
endif

# to improve SBC performance
LOCAL_CFLAGS:= -funroll-loops
LOCAL_CFLAGS+= -DBITPOOL_PROP

LOCAL_C_INCLUDES:= \
	$(LOCAL_PATH)/../sbc \
	../../../../frameworks/base/include \
	system/bluetooth/bluez-clean-headers

LOCAL_SHARED_LIBRARIES := \
	libcutils

ifneq ($(wildcard system/bluetooth/legacy.mk),)
LOCAL_STATIC_LIBRARIES := \
	libpower

LOCAL_MODULE := liba2dp
else
LOCAL_SHARED_LIBRARIES += \
	libpower \
	libdl

ifneq (,$(filter userdebug eng,$(TARGET_BUILD_VARIANT)))
LOCAL_CFLAGS    += -DENABLE_DATA_DUMP
LOCAL_SRC_FILES += android_audio_hw_icx_dump.c
endif

ifeq ($(ICX_FEATURE_HIGHRES),1)
LOCAL_CFLAGS+= -DICX_FEATURE_HIGHRES
endif


$(call import-module,cpufeatures)

LOCAL_MODULE := audio.a2dp.default
LOCAL_MODULE_PATH := $(TARGET_OUT_SHARED_LIBRARIES)/hw
endif

LOCAL_MODULE_TAGS := optional

include $(BUILD_SHARED_LIBRARY)


#-------------------------------------------------------------------------------
#
# libsrc (default)
#
ifneq ($(findstring icx, $(TARGET_PRODUCT)), )

include $(CLEAR_VARS)

LOCAL_SRC_FILES  := libsrc_default.c

LOCAL_SHARED_LIBRARIES := libcutils

LOCAL_MODULE      := libsrc.default
LOCAL_MODULE_PATH := $(TARGET_OUT_SHARED_LIBRARIES)/hw
LOCAL_MODULE_TAGS := optional

include $(BUILD_SHARED_LIBRARY)

endif


#-------------------------------------------------------------------------------
#
# libmixer (default)
#
ifneq ($(findstring icx, $(TARGET_PRODUCT)), )

include $(CLEAR_VARS)

LOCAL_SRC_FILES  := libmix_default.c

LOCAL_SHARED_LIBRARIES := libcutils

LOCAL_MODULE      := libmix.default
LOCAL_MODULE_PATH := $(TARGET_OUT_SHARED_LIBRARIES)/hw
LOCAL_MODULE_TAGS := optional

include $(BUILD_SHARED_LIBRARY)

endif


#-------------------------------------------------------------------------------
#
# pcm_bluetooth (alsa plugin)
#
include $(CLEAR_VARS)

LOCAL_CFLAGS := \
	-fPIC -DPIC -D_POSIX_SOURCE \
	-DALSA_CONFIG_DIR=\"/system/usr/share/alsa\" \
	-DALSA_PLUGIN_DIR=\"/system/usr/lib/alsa-lib\"

# to improve SBC performance
LOCAL_CFLAGS += -funroll-loops

LOCAL_C_INCLUDES:= \
	$(LOCAL_PATH)/include \
	$(LOCAL_PATH)/android \
	external/alsa-lib/include \
	$(LOCAL_PATH)/../sbc

LOCAL_SRC_FILES:= \
	pcm_bluetooth.c \
	ipc.c \
	../sbc/sbc_primitives.c \
	../sbc/sbc_primitives_neon.c

ifeq ($(TARGET_ARCH),x86)
LOCAL_SRC_FILES+= \
	../sbc/sbc_primitives_mmx.c \
	../sbc/sbc.c
else
LOCAL_SRC_FILES+= \
	../sbc/sbc.c.arm \
	../sbc/sbc_primitives_armv6.c
endif

LOCAL_MODULE := libasound_module_pcm_bluetooth
LOCAL_MODULE_PATH := $(TARGET_OUT_SHARED_LIBRARIES)/alsa-lib
LOCAL_MODULE_TAGS := optional

LOCAL_SHARED_LIBRARIES := \
	libasound \
	libc

include $(BUILD_SHARED_LIBRARY)
