/*
 *  Copyright (C) 2008-2011 The Android Open Source Project
 *  Copyright 2013 Sony Corporation
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */
/*
 * ChangeLog:
 *   2012-2013 changed by Sony Corporation
 */
#ifndef ANDROID_AUDIO_HW_ICX_BUFFER_H_
#define ANDROID_AUDIO_HW_ICX_BUFFER_H_

/*
 * PCM Buffer control class.
 * NOTE: supports only 32bit 2ch PCM for now.
 */
/*----------------------------------------------------------------------------*/
#include <stdint.h>
#include <stdlib.h>
#include <stdbool.h>
#include <pthread.h>

#include "android_audio_hw_icx_common.h"

/*----------------------------------------------------------------------------*/
#define NUM_OF_PERIOD_BUFFER 6

#define ICXB_status int
#define ICXB_SUCCESS 0
#define ICXB_ERROR   -1

/*----------------------------------------------------------------------------*/
typedef struct ICXBuffer_ ICXBuffer;
struct ICXBuffer_
{
    pthread_mutex_t lock;
    pthread_cond_t  cond;

    size_t   size_of_a_frame;
    uint32_t sample_rate_of_frame;
    size_t   frame_num_in_period_buffer;

    uint64_t* buf;
    size_t    frame_num_in_buf;
    size_t    written_frame_num;

    size_t    buf_read_index;
    size_t    buf_write_index;

    bool is_read_empty;
    bool is_write_empty;
};

/*----------------------------------------------------------------------------*/
ICXB_status ICXBuffer_initialize(ICXBuffer* self, size_t frame_num_in_period_buffer);
void        ICXBuffer_finalize(ICXBuffer* self);
void        ICXBuffer_reset(ICXBuffer* self);

/*----------------------------------------------------------------------------*/
size_t ICXBuffer_get_period_buffer_size(const ICXBuffer* self);
size_t ICXBuffer_get_period_buffer_frame_num(const ICXBuffer* self);
size_t ICXBuffer_get_buffer_frame_num(const ICXBuffer* self);

/*----------------------------------------------------------------------------*/
uint32_t ICXB_get_writable_frame_num(ICXBuffer* self, bool lock);
uint32_t ICXB_get_readable_frame_num(ICXBuffer* self, bool lock);

ICXB_status ICXB_write(ICXBuffer* self, size_t frame_num, uint64_t* buf, bool lock);
ICXB_status ICXB_read(ICXBuffer* self,  size_t frame_num, uint64_t* buf, bool lock);
ICXB_status ICXB_increment_read_index(ICXBuffer* self, size_t frame_num, bool lock);
ICXB_status ICXB_wait_writable(ICXBuffer* self, uint32_t timeout_ms, bool lock);

/*----------------------------------------------------------------------------*/
void ICXB_lock(ICXBuffer* self);
void ICXB_unlock(ICXBuffer* self);


#endif  /* ANDROID_AUDIO_HW_ICX_BUFFER_H_ */
