/*
 *  Copyright 2013 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU Lesser General Public License as published by
 *  the Free Software Foundation;  version 2.1 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU Lesser General Public License
 *  along with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 *
 */
#ifndef ANDROID_AUDIO_HW_ICX_COMMON_H_
#define ANDROID_AUDIO_HW_ICX_COMMON_H_


/*----------------------------------------------------------------------------*/
#include "unistd.h"

/*----------------------------------------------------------------------------*/
#define A2DP_WAKE_LOCK_NAME "A2dpOutputStream"

/*----------------------------------------------------------------------------*/
/* frame num in ICXBuffer period.
   NOTE: period frame num must be multiple of SRC_PROCESSING_FRAME_NUM
         to avoid difficulty of processing SRC. */
#define ICX_BUFFER_PERIOD_FRAME_NUM (512 * 4)

/* a2dp_write() needs at least 512 bytes (16bit PCM) = 128 frames = 256 sample
   See comment in liba2dp.c::a2dp_write(). */
#define A2DP_WRITE_MIN_FRAME_NUM (128)

/* thread's minimum processasble frame num. */
#define PROCESSABLE_MIN_FRAME_NUM               \
    ( MAX( A2DP_WRITE_MIN_FRAME_NUM, MIX_PROCESSING_FRAME_NUM ) )

/*----------------------------------------------------------------------------*/
#define CH_NUM_OF_A_FRAME       (2) /* 2ch in a frame       */
#define SIZE_OF_16BIT_PCM_FRAME (4) /* = 16bit sample * 2ch */
#define SIZE_OF_32BIT_PCM_FRAME (8) /* = 32bit sample * 2ch */

/*----------------------------------------------------------------------------*/
typedef enum {
    STYPE_STD = 0,
    STYPE_ICX
} stream_type;

/*-------------------------------------------------------------- util macro --*/
#define UPTO(val, max)    ((val) > (max))?(max):(val);
#define MAX(a, b)         (((a) > (b))?(a):(b))
#define MIN(a, b)         (((a) > (b))?(b):(a))
#define SIZE_OF_ARRAY(ar) (sizeof(ar) / sizeof(ar[0]))

#define HAL_ASSERT(expect_true)                             \
    if (!(expect_true)) {                                   \
        ALOGE("---- CHECK THIS ---- => (%s)(%s:%d)", #expect_true, __FILE__, __LINE__); \
    }

/* Extract value which corresponds to key from str_parms_ptr */
#define GET_VALUE_FROM_STR_PARMS(str_parms_ptr, key, value_buf)       \
    str_parms_get_str((str_parms_ptr), (key), (value_buf), sizeof(value_buf))

/*--------------------------------------------------------------- util func --*/
#include <sys/time.h>
#include <time.h>

/* returns in nsec */
static uint64_t system_time(void)
{
    struct timespec t;

    t.tv_sec = t.tv_nsec = 0;
    clock_gettime(CLOCK_MONOTONIC, &t);

    return t.tv_sec*1000000000LL + t.tv_nsec;
}

/* returns in usec */
static uint32_t now_us()
{
    return system_time() / 1000UL;
}

static uint32_t
frame_num_to_msec(uint32_t frame_num, uint32_t sampling_rate)
{
    return ((frame_num * 1000) / sampling_rate);
}

/* returns in usec */
static uint32_t frame_num_to_duration(uint32_t frame_num, uint32_t sampling_rate)
{
    return frame_num_to_msec(frame_num, sampling_rate) * 1000;
}

static uint32_t duration_to_frame_num(uint32_t duration_us, uint32_t sampling_rate)
{
    return (duration_us * sampling_rate) / 1000 / 1000;
}

static void
usleep_by_frame_num(uint32_t frame_num, uint32_t sampling_rate)
{
    usleep( frame_num_to_msec(frame_num, sampling_rate) * 1000 );
}


#endif  /* ANDROID_AUDIO_HW_ICX_COMMON_H_ */
