/*
 *  Copyright 2013 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU Lesser General Public License as published by
 *  the Free Software Foundation;  version 2.1 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU Lesser General Public License
 *  along with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 *
 */
#ifndef ANDROID_AUDIO_HW_ICX_DEBUG_H_
#define ANDROID_AUDIO_HW_ICX_DEBUG_H_

/*----------------------------------------------------------------------------*/
#define CASE_RETURN_ITS_STR(value)              \
    case (value):                               \
    return "" #value;                           \
    break;                                      \

/* see system/core/include/system/audio.h */
static const char*
debug_print_audio_format(audio_format_t format)
{
    switch (format)
    {
        CASE_RETURN_ITS_STR(AUDIO_FORMAT_INVALID);
        CASE_RETURN_ITS_STR(AUDIO_FORMAT_PCM);
        CASE_RETURN_ITS_STR(AUDIO_FORMAT_MP3);
        CASE_RETURN_ITS_STR(AUDIO_FORMAT_AMR_NB);
        CASE_RETURN_ITS_STR(AUDIO_FORMAT_AMR_WB);
        CASE_RETURN_ITS_STR(AUDIO_FORMAT_AAC);
        CASE_RETURN_ITS_STR(AUDIO_FORMAT_HE_AAC_V1);
        CASE_RETURN_ITS_STR(AUDIO_FORMAT_HE_AAC_V2);
        CASE_RETURN_ITS_STR(AUDIO_FORMAT_VORBIS);
        CASE_RETURN_ITS_STR(AUDIO_FORMAT_MAIN_MASK);
        CASE_RETURN_ITS_STR(AUDIO_FORMAT_SUB_MASK);
        CASE_RETURN_ITS_STR(AUDIO_FORMAT_PCM_16_BIT);
        CASE_RETURN_ITS_STR(AUDIO_FORMAT_PCM_8_BIT);
        CASE_RETURN_ITS_STR(AUDIO_FORMAT_PCM_32_BIT);
        CASE_RETURN_ITS_STR(AUDIO_FORMAT_PCM_8_24_BIT);
        CASE_RETURN_ITS_STR(AUDIO_FORMAT_PCM_24_BIT);
    }

    return "unknown";
};

/*----------------------------------------------------------------------------*/
/* see system/core/include/system/audio.h  */
static const char*
debug_print_channel_mask(audio_channel_mask_t mask)
{
    switch (mask)
    {
        /* CASE_RETURN_ITS_STR(AUDIO_CHANNEL_OUT_FRONT_LEFT); */
        CASE_RETURN_ITS_STR(AUDIO_CHANNEL_OUT_FRONT_RIGHT);
        /* CASE_RETURN_ITS_STR(AUDIO_CHANNEL_OUT_FRONT_CENTER); */
        CASE_RETURN_ITS_STR(AUDIO_CHANNEL_OUT_LOW_FREQUENCY);
        CASE_RETURN_ITS_STR(AUDIO_CHANNEL_OUT_BACK_LEFT);
        CASE_RETURN_ITS_STR(AUDIO_CHANNEL_OUT_BACK_RIGHT);
        CASE_RETURN_ITS_STR(AUDIO_CHANNEL_OUT_FRONT_LEFT_OF_CENTER);
        CASE_RETURN_ITS_STR(AUDIO_CHANNEL_OUT_FRONT_RIGHT_OF_CENTER);
        CASE_RETURN_ITS_STR(AUDIO_CHANNEL_OUT_BACK_CENTER);
        CASE_RETURN_ITS_STR(AUDIO_CHANNEL_OUT_SIDE_LEFT);
        CASE_RETURN_ITS_STR(AUDIO_CHANNEL_OUT_SIDE_RIGHT);
        CASE_RETURN_ITS_STR(AUDIO_CHANNEL_OUT_TOP_CENTER);
        CASE_RETURN_ITS_STR(AUDIO_CHANNEL_OUT_TOP_FRONT_LEFT);
        CASE_RETURN_ITS_STR(AUDIO_CHANNEL_OUT_TOP_FRONT_CENTER);
        CASE_RETURN_ITS_STR(AUDIO_CHANNEL_OUT_TOP_FRONT_RIGHT);
        CASE_RETURN_ITS_STR(AUDIO_CHANNEL_OUT_TOP_BACK_LEFT);
        CASE_RETURN_ITS_STR(AUDIO_CHANNEL_OUT_TOP_BACK_CENTER);
        CASE_RETURN_ITS_STR(AUDIO_CHANNEL_OUT_TOP_BACK_RIGHT);
        CASE_RETURN_ITS_STR(AUDIO_CHANNEL_OUT_MONO);
        CASE_RETURN_ITS_STR(AUDIO_CHANNEL_OUT_STEREO);
        CASE_RETURN_ITS_STR(AUDIO_CHANNEL_OUT_QUAD);
        CASE_RETURN_ITS_STR(AUDIO_CHANNEL_OUT_SURROUND);
        CASE_RETURN_ITS_STR(AUDIO_CHANNEL_OUT_5POINT1);
        CASE_RETURN_ITS_STR(AUDIO_CHANNEL_OUT_7POINT1);
        CASE_RETURN_ITS_STR(AUDIO_CHANNEL_OUT_ALL);

        /* CASE_RETURN_ITS_STR(AUDIO_CHANNEL_IN_LEFT); */
        /* CASE_RETURN_ITS_STR(AUDIO_CHANNEL_IN_RIGHT); */
        /* CASE_RETURN_ITS_STR(AUDIO_CHANNEL_IN_FRONT); */
        /* CASE_RETURN_ITS_STR(AUDIO_CHANNEL_IN_BACK); */
        /* CASE_RETURN_ITS_STR(AUDIO_CHANNEL_IN_LEFT_PROCESSED); */
        /* CASE_RETURN_ITS_STR(AUDIO_CHANNEL_IN_RIGHT_PROCESSED); */
        /* CASE_RETURN_ITS_STR(AUDIO_CHANNEL_IN_FRONT_PROCESSED); */
        /* CASE_RETURN_ITS_STR(AUDIO_CHANNEL_IN_BACK_PROCESSED); */
        /* CASE_RETURN_ITS_STR(AUDIO_CHANNEL_IN_PRESSURE); */
        /* CASE_RETURN_ITS_STR(AUDIO_CHANNEL_IN_X_AXIS); */
        /* CASE_RETURN_ITS_STR(AUDIO_CHANNEL_IN_Y_AXIS); */
        /* CASE_RETURN_ITS_STR(AUDIO_CHANNEL_IN_Z_AXIS); */
        /* CASE_RETURN_ITS_STR(AUDIO_CHANNEL_IN_VOICE_UPLINK); */
        /* CASE_RETURN_ITS_STR(AUDIO_CHANNEL_IN_VOICE_DNLINK); */
        /* CASE_RETURN_ITS_STR(AUDIO_CHANNEL_IN_MONO); */
        /* CASE_RETURN_ITS_STR(AUDIO_CHANNEL_IN_STEREO); */
        /* CASE_RETURN_ITS_STR(AUDIO_CHANNEL_IN_ALL); */
    }
    return "others";
}

/*----------------------------------------------------------------------------*/
static const char*
debug_print_audio_output_flags(audio_output_flags_t flag)
{
    switch (flag)
    {
        CASE_RETURN_ITS_STR(AUDIO_OUTPUT_FLAG_NONE);
        CASE_RETURN_ITS_STR(AUDIO_OUTPUT_FLAG_DIRECT);
        CASE_RETURN_ITS_STR(AUDIO_OUTPUT_FLAG_PRIMARY);
        CASE_RETURN_ITS_STR(AUDIO_OUTPUT_FLAG_FAST);
        CASE_RETURN_ITS_STR(AUDIO_OUTPUT_FLAG_DEEP_BUFFER);
    }
    return "others";
}

/*----------------------------------------------------------------------------*/
/* see system/core/include/system/audio.h  */
static const char*
debug_print_audio_devices(audio_devices_t devices)
{
    switch (devices)
    {
        /* output devices */
        CASE_RETURN_ITS_STR(AUDIO_DEVICE_OUT_EARPIECE);
        CASE_RETURN_ITS_STR(AUDIO_DEVICE_OUT_SPEAKER);
        CASE_RETURN_ITS_STR(AUDIO_DEVICE_OUT_WIRED_HEADSET);
        CASE_RETURN_ITS_STR(AUDIO_DEVICE_OUT_WIRED_HEADPHONE);
        CASE_RETURN_ITS_STR(AUDIO_DEVICE_OUT_BLUETOOTH_SCO);
        CASE_RETURN_ITS_STR(AUDIO_DEVICE_OUT_BLUETOOTH_SCO_HEADSET);
        CASE_RETURN_ITS_STR(AUDIO_DEVICE_OUT_BLUETOOTH_SCO_CARKIT);
        CASE_RETURN_ITS_STR(AUDIO_DEVICE_OUT_BLUETOOTH_A2DP);
        CASE_RETURN_ITS_STR(AUDIO_DEVICE_OUT_BLUETOOTH_A2DP_HEADPHONES);
        CASE_RETURN_ITS_STR(AUDIO_DEVICE_OUT_BLUETOOTH_A2DP_SPEAKER);
        CASE_RETURN_ITS_STR(AUDIO_DEVICE_OUT_AUX_DIGITAL);
        CASE_RETURN_ITS_STR(AUDIO_DEVICE_OUT_ANLG_DOCK_HEADSET);
        CASE_RETURN_ITS_STR(AUDIO_DEVICE_OUT_DGTL_DOCK_HEADSET);
        CASE_RETURN_ITS_STR(AUDIO_DEVICE_OUT_USB_ACCESSORY);
        CASE_RETURN_ITS_STR(AUDIO_DEVICE_OUT_USB_DEVICE);
#ifdef OMAP_ENHANCEMENT
        CASE_RETURN_ITS_STR(AUDIO_DEVICE_OUT_FM_RADIO_TX);
#endif
        CASE_RETURN_ITS_STR(AUDIO_DEVICE_OUT_WMPORT);
        CASE_RETURN_ITS_STR(AUDIO_DEVICE_OUT_WMPORTFIXED);
        CASE_RETURN_ITS_STR(AUDIO_DEVICE_OUT_DEFAULT);
        CASE_RETURN_ITS_STR(AUDIO_DEVICE_OUT_ALL);
        CASE_RETURN_ITS_STR(AUDIO_DEVICE_OUT_ALL_A2DP);
        CASE_RETURN_ITS_STR(AUDIO_DEVICE_OUT_ALL_SCO);
        CASE_RETURN_ITS_STR(AUDIO_DEVICE_OUT_ALL_USB);

    default:
        return "may be input device";
        break;
    }
    return "others";
}

/*----------------------------------------------------------------------------*/
/* see system/core/include/system/audio.h  */
static const char*
debug_print_audio_mode(audio_mode_t mode)
{
    switch (mode)
    {
        CASE_RETURN_ITS_STR(AUDIO_MODE_INVALID);
        CASE_RETURN_ITS_STR(AUDIO_MODE_CURRENT);
        CASE_RETURN_ITS_STR(AUDIO_MODE_NORMAL);
        CASE_RETURN_ITS_STR(AUDIO_MODE_RINGTONE);
        CASE_RETURN_ITS_STR(AUDIO_MODE_IN_CALL);
        CASE_RETURN_ITS_STR(AUDIO_MODE_IN_COMMUNICATION);
    default:
        return "invalid";
        break;
    }
    return "not found";
}

#endif  /* ANDROID_AUDIO_HW_ICX_DEBUG_H_ */
