/*
 *  Copyright 2013 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU Lesser General Public License as published by
 *  the Free Software Foundation;  version 2.1 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU Lesser General Public License
 *  along with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 *
 */
#include "android_audio_hw_icx_mixer.h"
#include "android_audio_hw_icx_common.h"

#define LOG_TAG "a2dp_audio_hw_mixer"
/* #define LOG_NDEBUG 0 */
#include <cutils/log.h>


/*----------------------------------------------------------------------------*/
#include <dlfcn.h>
#include <errno.h>
#include "libmix.h"

/*----------------------------------------------------------------------------*/
#define SO_PATH_ICX     "/system/lib/hw/libmix.icx.so"
#define SO_PATH_DEFAULT "/system/lib/hw/libmix.default.so"

static const char* s_pathes[] = { SO_PATH_ICX, SO_PATH_DEFAULT };

/*----------------------------------------------------------------------------*/
static struct libmix_handle* s_lmh = NULL;
static struct libmix*        s_lm  = NULL;

/*----------------------------------------------------------------------------*/
static void*
open_so(const char* path)
{
    ALOGI("open so %s", path);

    void* handle = dlopen(path, RTLD_NOW);
    if (handle == NULL)
    {
        char const *err_str = dlerror();
        ALOGE("load: module = %s\n%s", path, err_str ? err_str : "unknown");
        return NULL;
    }
    return handle;
}

static int
open_lib(struct libmix_handle** plmh)
{
    uint  index  = 0;
    void* handle = NULL;
    do
    {
        handle = open_so(s_pathes[index++]);
    } while (handle == NULL && index < SIZE_OF_ARRAY(s_pathes));

    if (handle == NULL) { return -1; }


    *plmh = (struct libmix_handle*)dlsym(handle, LIB_MIX_SYMBOL_NAME);
    if (*plmh == NULL)
    {
        ALOGE("load: couldn't find symobol %s", LIB_MIX_SYMBOL_NAME);
        dlclose(handle);
        return -1;
    }

    (*plmh)->dso = handle;

    return 0;
}

static void
close_lib(struct libmix_handle* lmh)
{
    if (lmh != NULL)
    {
        dlclose(lmh->dso);
        lmh = NULL;
    }
}

/*----------------------------------------------------------------------------*/
int
MW_initialize(void)
{
    HAL_ASSERT(s_lm  == NULL);
    HAL_ASSERT(s_lmh == NULL);

    int ret;
    if (s_lmh == NULL)
    {
        ret = open_lib(&s_lmh);
        if (ret != 0)
        {
            ALOGE("failed open_lib");
            return -1;
        }
    }

    HAL_ASSERT(s_lmh != NULL);
    ret = s_lmh->get_libmix(&s_lm);
    if (ret != 0)
    {
        ALOGE("failed get_libmix");
        return -1;
    }

    ret = s_lm->initialize(s_lm, 44100, MIX_PROCESSING_FRAME_NUM);
    if (ret != 0)
    {
        ALOGE("libmix->initialize() failed");
        return -1;
    }

    return 0;
}

int
MW_finalize()
{
    if (s_lm != NULL)
    {
        s_lm->finalize(s_lm);
        s_lmh->release_libmix(s_lm);
    }
    s_lm = NULL;

    close_lib(s_lmh);
    return 0;
}

uint32_t
MW_mix(uint64_t* input_icx,
       uint64_t* input_std,
       uint64_t* out_buf,
       uint32_t  count)
{
    HAL_ASSERT(s_lm != NULL);
    return s_lm->mix(s_lm, input_icx, input_std, out_buf, count);
}


