/*
 *  Copyright 2013 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU Lesser General Public License as published by
 *  the Free Software Foundation;  version 2.1 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU Lesser General Public License
 *  along with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 *
 */
#include "android_audio_hw_icx_src.h"
#include "android_audio_hw_icx_common.h"

#define LOG_TAG "a2dp_audio_hw_src"
/* #define LOG_NDEBUG 0 */
#include <cutils/log.h>


/*----------------------------------------------------------------------------*/
#include <dlfcn.h>
#include <errno.h>
#include "libsrc.h"

/*----------------------------------------------------------------------------*/
#define SO_PATH_ICX     "/system/lib/hw/libsrc.icx.so"
#define SO_PATH_DEFAULT "/system/lib/hw/libsrc.default.so"

static const char* s_pathes[] = { SO_PATH_ICX, SO_PATH_DEFAULT };

/*----------------------------------------------------------------------------*/
static struct libsrc_handle* s_lsh = NULL;
static struct libsrc*        s_ls  = NULL;

/*----------------------------------------------------------------------------*/
static void*
open_so(const char* path)
{
    ALOGI("open so %s", path);

    void* handle = dlopen(path, RTLD_NOW);
    if (handle == NULL)
    {
        char const *err_str = dlerror();
        ALOGE("load: module = %s\n%s", path, err_str ? err_str : "unknown");
        return NULL;
    }
    return handle;
}

static int
open_lib(struct libsrc_handle** plsh)
{
    uint  index  = 0;
    void* handle = NULL;
    do
    {
        handle = open_so(s_pathes[index++]);
    } while (handle == NULL && index < SIZE_OF_ARRAY(s_pathes));

    if (handle == NULL) { return -1; }


    *plsh = (struct libsrc_handle*)dlsym(handle, LIB_SRC_SYMBOL_NAME);
    if (*plsh == NULL)
    {
        ALOGE("load: couldn't find symobol %s", LIB_SRC_SYMBOL_NAME);
        dlclose(handle);
        return -1;
    }

    (*plsh)->dso = handle;

    return 0;
}

static void
close_lib(struct libsrc_handle* lsh)
{
    if (lsh != NULL)
    {
        dlclose(lsh->dso);
        lsh = NULL;
    }
}

/*----------------------------------------------------------------------------*/
int
SW_open(void)
{
    HAL_ASSERT(s_ls  == NULL);
    HAL_ASSERT(s_lsh == NULL);

    int ret;
    ret = open_lib(&s_lsh);
    if (ret != 0)
    {
        ALOGE("failed open_lib");
        return -1;
    }

    HAL_ASSERT(s_lsh != NULL);
    ret = s_lsh->get_libsrc(&s_ls);
    if (ret != 0)
    {
        ALOGE("failed get_libsrc");
        return -1;
    }

    return 0;
}

void
SW_close(void)
{
    HAL_ASSERT(s_lsh != NULL);

    if (s_ls != NULL)
    {
        s_lsh->release_libsrc(s_ls);
    }
    s_ls = NULL;

    close_lib(s_lsh);

    HAL_ASSERT(s_lsh == NULL);
}

/*----------------------------------------------------------------------------*/
int
SW_initialize(uint32_t input_sample_rate,
              uint32_t output_sample_rate)
{
    HAL_ASSERT(s_ls != NULL);
    int ret = s_ls->initialize(s_ls,
                               input_sample_rate,
                               output_sample_rate,
                               SRC_PROCESSING_FRAME_NUM);
    if (ret != 0)
    {
        ALOGE("libsrc->initialize() failed");
        return -1;
    }

    return 0;
}

void
SW_finalize(void)
{
    HAL_ASSERT(s_ls != NULL);
    s_ls->finalize(s_ls);
}

int
SW_get_max_output_frame_num(void)
{
    HAL_ASSERT(s_ls != NULL);
    return s_ls->get_max_output_frame_num(s_ls);
}

int
SW_convert(const uint64_t* input_buf,
           uint64_t*       output_buf,
           int*            output_frame_num)
{
    HAL_ASSERT(s_ls != NULL);
    return s_ls->convert(s_ls, input_buf, output_buf, output_frame_num);
}
