/*
 *  Copyright (C) 2008-2011 The Android Open Source Project
 *  Copyright 2013 Sony Corporation
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */
/*
 * ChangeLog:
 *   2012-2013 changed by Sony Corporation
 */
#ifndef ANDROID_AUDIO_HW_ICX_STREAM_H_
#define ANDROID_AUDIO_HW_ICX_STREAM_H_

/*----------------------------------------------------------------------------*/
#include <hardware/audio.h>

#include "android_audio_hw_icx.h"
#include "android_audio_hw_icx_buffer.h"

/*----------------------------------------------------------------------------*/
typedef struct stream_out_ stream_out;

/*----------------------------------------------------------------------------*/
typedef enum
{
    SS_NOT_USED = 0,
    SS_WAIT_PLAYING,
    SS_PLAYING,
    SS_WAIT_CLOSING,
    SS_STATE_NUM
} stream_state;

/*----------------------------------------------------------------------------*/
stream_state sout_get_state(stream_out* out);
bool         sout_is_standby(stream_out* out);
void         sout_set_standby(stream_out* out, bool value);

/*----------------------------------------------------------------------------*/
int open_stream_std(stream_out**         stream_out_ptr,
                    audio_device**       parent,
                    audio_devices_t      devices,
                    struct audio_config* config,
                    ICXBuffer*           buf);
int open_stream_icx(stream_out**         stream_out_ptr,
                    audio_device**       parent,
                    audio_devices_t      devices,
                    struct audio_config* config,
                    ICXBuffer*           buf);

void close_stream_std(stream_out* out);
void close_stream_icx(stream_out* out);

/*----------------------------------------------------------------------------*/
struct audio_stream_out* sout_get_audio_stream_out(stream_out* out);
void sout_set_fade(stream_out* out, uint32_t mute_time_ms);
void sout_flush_mute(stream_out* out);


#endif  /* ANDROID_AUDIO_HW_ICX_STREAM_H_ */
