/*
 *  Copyright 2013 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU Lesser General Public License as published by
 *  the Free Software Foundation;  version 2.1 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU Lesser General Public License
 *  along with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 *
 */

#ifndef _AUXAUDIO_COMMON_
#define _AUXAUDIO_COMMON_

/* #define ENTRY_PRINT_ON */
/* #define TRACE_PRINT_ON */
/* #define DEBUG_PRINT_ON */

#define LOG_TAG "AuxAudio"
#define LOG_NDEBUG 0

#define BACK_TRACE_ON


/* suported audio parameters */

#define MAX_SUPPORTED_RATE 6
#define MAX_SUPPORTED_BITS 3
#define MAX_SUPPORTED_CH   2

static const int rate_choice_table[MAX_SUPPORTED_RATE][MAX_SUPPORTED_RATE]={
	{  44100,  88200, 176400, 192000,  96000,  48000},
	{  88200, 176400, 192000,  96000,  44100,  48000},
	{ 176400, 192000,  88200,  96000,  44100,  48000},
	{  48000,  96000, 192000, 176400,  88200,  44100},
	{  96000, 192000, 176400,  88200,  48000,  44100},
	{ 192000, 176400,  96000,  88200,  48000,  44100},
};

/* thread priority */
#define CORE_THREAD_PRIORITY     50
#define ANDROID_PRIORITY_AUDIO (-19)

/* time */
#define DATA_WAIT_TIMEOUT           30 /* ms */
#define DATA_PREPARE_WAIT           50 /* ms */
#define SLEEP_TIME_BOTH_NO_ACTION 5000 /* ms */

/* FIFO size */
#define FIFO_MULTIPLE 5

/* SRC parameters */
#define SRC_FILTER_DECRESE_AMOUNT 100
#define SRC_CUTOFF_RATIO            0.91f

/* mixer parameters */
#define MIXER_ATTACK_TIME_CONST   0.001f
#define MIXER_RELEASE_TIME_CONST  0.5f
#define MIXER_THRESHOLD_DB       -6.02f

/* mute parameters [ms] */
/* #define STREAM_FADE_TIME 50 */
#define STREAM_FADE_TIME 2000

/* wave file parameters */
#define WAV_FILE_PATH "/data/misc/camera/AUX"
#define WAV_WRITE_SIZE 32768

/* fixed parameters */
#define CHANNEL_COUNT     2
#define SAMPLES_PER_FRAME 2
#define BYTE_PER_SAMPLE   4
#define BYTE_PER_FRAME    8

#include <errno.h>
#include <stdint.h>
#include <stdlib.h>
#include <fcntl.h>
#include <pthread.h>
#include <sys/time.h>
#include <cutils/log.h>
#include <cutils/str_parms.h>

#endif
