/*
 *  Copyright 2013 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU Lesser General Public License as published by
 *  the Free Software Foundation;  version 2.1 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU Lesser General Public License
 *  along with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 *
 */

/* #define TRACE_PRINT_ON */
/* #define DEBUG_PRINT_ON */
#define TRACE_TAG "        -------- "
#define DEBUG_TAG "                     "

#include "./common.h"

#include "misc.h"

int pthread_create_easy(
	pthread_t * handle,
	int         priority,
	void *      (*entry)(void *),
	void *      arg
)
{
	pthread_attr_t attr;
	struct sched_param sp;
	int rv;

	print_trace("%s(%d)",__FUNCTION__,priority);

	rv=pthread_attr_init(&attr);
	if(rv<0){
		print_fail("pthrad_attr_init(): %s",strerror(-rv));
		back_trace();
		return(rv);
	}

	/* rv=pthread_attr_setdetachstate(&attr,PTHREAD_CREATE_DETACHED); */
	/* if(rv<0){ */
		/* print_fail("pthread_attr_setdetachstate(): %s",strerror(-rv)); */
		/* back_trace(); */
		/* pthread_attr_destroy(&attr); */
		/* return(rv); */
	/* } */

	rv=pthread_attr_setschedpolicy(&attr,SCHED_FIFO);
	if(rv<0){
		print_fail("pthread_attr_setschedpolicy(): %s",strerror(-rv));
		back_trace();
		pthread_attr_destroy(&attr);
		return(rv);
	}

	sp.sched_priority=priority;
	rv=pthread_attr_setschedparam(&attr,&sp);
	if(rv<0){
		print_fail("pthread_attr_setschedparam(): %s",strerror(-rv));
		back_trace();
		pthread_attr_destroy(&attr);
		return(rv);
	}

	rv=pthread_create(
		handle,
		/* &attr, */
		NULL,
		entry,
		arg
	);
	if(rv<0){
		print_fail("pthread_create(): %s",strerror(-rv));
		back_trace();
		pthread_attr_destroy(&attr);
		return(rv);
	}

	pthread_attr_destroy(&attr);

	return(0);
}

int get_suitable_frame_count(int rate)
{
	int count=1024;

	print_trace("%s(%d)",__FUNCTION__,rate);

	switch(rate){
		case  44100: count = 1024; break;
		case  88200: count = 2048; break;
		case 176400: count = 4096; break;
		case  48000: count = 1024; break;
		case  96000: count = 2048; break;
		case 192000: count = 4096; break;
	}

	return(count);
}

void mem_32_to_16(void * buf_32, void * buf_16, int size)
{
	unsigned int   * p32;
	unsigned short * p16;

	print_trace("%s(%d)",__FUNCTION__,size);

	p32=(unsigned int *)buf_32;
	p16=(unsigned short *)buf_16;

	while(size>0){

		*p16=(unsigned short)((*p32)>>16);
		p32++;
		p16++;
		size--;
	}

	return;
}

void mem_16_to_32(void * buf_16, void * buf_32, int size)
{
	unsigned int   * p32;
	unsigned short * p16;

	print_trace("%s(%d)",__FUNCTION__,size);

	p32=(unsigned int *)buf_32;
	p16=(unsigned short *)buf_16;

	while(size>0){

		*p32=(unsigned int)(*p16)<<16;
		p32++;
		p16++;
		size--;
	}

	return;
}

uint64_t system_time(void)
{
	struct timespec t;

	t.tv_sec  = 0;
	t.tv_nsec = 0;

	clock_gettime(CLOCK_MONOTONIC, &t);

	/* print_debug("%lu,%lu",t.tv_sec,t.tv_nsec); */

	return(t.tv_sec*1000000000LL + t.tv_nsec);
}

