/*
 *  Copyright 2013 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU Lesser General Public License as published by
 *  the Free Software Foundation;  version 2.1 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU Lesser General Public License
 *  along with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 *
 */

#ifndef _AUXAUDIO_MISC_
#define _AUXAUDIO_MISC_

#ifdef ENTRY_PRINT_ON
#define print_entry(fmt, args ...)  ALOGV(ENTRY_TAG fmt, ##args)
#else
#define print_entry(fmt, args ...)
#endif

#ifdef TRACE_PRINT_ON
#define print_trace(fmt, args ...) ALOGV(TRACE_TAG fmt, ##args)
#else
#define print_trace(fmt, args ...)
#endif

#ifdef DEBUG_PRINT_ON
#define print_debug(fmt, args ...) ALOGD(DEBUG_TAG "%s(): " fmt, __FUNCTION__, ##args)
#else
#define print_debug(fmt, args ...)
#endif

#ifdef BACK_TRACE_ON
#define back_trace() ALOGE("%s(): [%d]",__FUNCTION__,__LINE__)
#else
#define back_trace()
#endif

#define print_info(fmt, args ...)    ALOGI(         fmt,               ##args)
#define print_warning(fmt, args ...) ALOGW(         fmt,               ##args)
#define print_error(fmt, args ...)   ALOGE("%s(): " fmt, __FUNCTION__, ##args)
#define print_fail(fmt, args...)     ALOGE(         fmt,               ##args)

#define minimum(_a,_b) ((_a)<(_b)?(_a):(_b))
#define maximum(_a,_b) ((_a)>(_b)?(_a):(_b))

#define OTHSIDE(_a) ((_a)==STREAM_STD?STREAM_ICX:STREAM_STD)

#define FALSE 0
#define TRUE  1

#define STREAM_STD 0
#define STREAM_ICX 1

int pthread_create_easy(
	pthread_t * handle,
	int priority,
	void *(*entry)(void *),
	void * arg
);

int get_suitable_frame_count(int rate);

void mem_32_to_16(void * buf_32, void * buf_16, int size);
void mem_16_to_32(void * buf_16, void * buf_32, int size);

uint64_t system_time(void);

#endif
