/*
 *  Copyright 2013 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU Lesser General Public License as published by
 *  the Free Software Foundation;  version 2.1 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU Lesser General Public License
 *  along with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 *
 */

#ifndef _AUXAUDIO_TIMED_MUTE_
#define _AUXAUDIO_TIMED_MUTE_

#include <pthread.h>

struct timed_mute_handle {
	char * name;
	pthread_mutex_t mutex;
	struct mute_handle * handle;

	int rate; /* Hz */
	int mute;
	int virgin;

	uint64_t base;    /* nsec */
	uint64_t timeout; /* nsec */

	int remain; /* samples */
};

struct timed_mute_handle * timed_mute_open(
	char * name,
	int    rate,
	int    fade
);

void timed_mute_close(
	struct timed_mute_handle * handle
);

int timed_mute_set_rate(
	struct timed_mute_handle * handle,
	int                        rate
);

int timed_mute_switch(
	struct timed_mute_handle * handle,
	int                        value
);

int timed_mute_attenuate(
	struct timed_mute_handle * handle,
	int *                      buffer,
	int                        samples
);

int timed_mute_flush(
	struct timed_mute_handle * handle
);

#endif
