/*
 *  Copyright 2013 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU Lesser General Public License as published by
 *  the Free Software Foundation;  version 2.1 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU Lesser General Public License
 *  along with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 *
 */
#include "libmix.h"

#define LOG_TAG LIB_MIX_LOG_TAG_NAME
/* #define LOG_NDEBUG 0 */
#include <cutils/log.h>

/*----------------------------------------------------------------------------*/
#define SIZE_OF_32BIT_PCM_FRAME (8)

/*----------------------------------------------------------------------------*/
typedef struct MixerWrapper_
{
    struct libmix super;
} MixerWrapper;

/*----------------------------------------------------------------------------*/
static int
MW_initialize(struct libmix* lm,
              uint32_t       sampling_rate,
              uint32_t       processing_frame_num)
{
    return 0;
}

static int
MW_finalize(struct libmix* lm)
{
    return 0;
}

/*
 * Reads a frame (32 bit 2ch PCM only)
 * and separates into left and right.
 */
static void
read_one_frame(void*     read_addr,
               uint32_t* left,
               uint32_t* right)
{
    uint64_t tmp64;
    memcpy(&tmp64, read_addr, SIZE_OF_32BIT_PCM_FRAME);
    *left  = (tmp64 >> 32);
    *right = (tmp64 << 32) >> 32;
}

static uint32_t
MW_mix2(struct libmix* lm,
        uint64_t*      input_icx,
        uint64_t*      input_std,
        uint64_t*      out_buf,
        uint32_t       count)
{
    MixerWrapper* mw = (MixerWrapper*)lm;

    uint i;

    uint32_t icx_sample_l;
    uint32_t icx_sample_r;

    uint32_t std_sample_l;
    uint32_t std_sample_r;

    uint32_t sample_left_32;
    uint32_t sample_right_32;

    uint64_t tmp;

    for (i = 0; i < count; i++)
    {
        icx_sample_l = 0;
        icx_sample_r = 0;
        std_sample_l  = 0;
        std_sample_r  = 0;

        read_one_frame((void*)(input_icx + i), &icx_sample_l, &icx_sample_r);
        read_one_frame((void*)(input_std + i), &std_sample_l, &std_sample_r);

        /* Simple Mix */
        sample_left_32  = (((icx_sample_l >> 1) + (std_sample_l >> 1)) << 1); /* >> 1 means 6db down */
        sample_right_32 = (((icx_sample_r >> 1) + (std_sample_r >> 1)) << 1); /* >> 1 means 6db down */

        tmp = sample_right_32;
        out_buf[i] = ((tmp << 32 ) | sample_right_32);
    }

    return count;
}


/*----------------------------------------------------------------------------*/
static int
get_libmix(struct libmix** plm)
{
    MixerWrapper* mw = (MixerWrapper*)malloc(sizeof(MixerWrapper));
    if (mw == NULL)
    {
        ALOGE("malloc failed");
        return -1;
    }

    mw->super.initialize = MW_initialize;
    mw->super.finalize   = MW_finalize;
    mw->super.mix        = MW_mix2;

    *plm = &mw->super;
    return 0;
}

static void
release_libmix(struct libmix* lm)
{
    MixerWrapper* mw = (MixerWrapper*)lm;
    free(mw);
}

/*----------------------------------------------------------------------------*/
struct libmix_handle HAL_LIB_MIX = {
    .get_libmix     = get_libmix,
    .release_libmix = release_libmix
};


