/*
 *  Copyright 2013 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU Lesser General Public License as published by
 *  the Free Software Foundation;  version 2.1 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU Lesser General Public License
 *  along with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 *
 */
#include "libsrc.h"

#define LOG_TAG LIB_SRC_LOG_TAG_NAME
/* #define LOG_NDEBUG 0 */
#include <cutils/log.h>

/*----------------------------------------------------------------------------*/
#define SIZE_OF_32BIT_PCM_FRAME (8)

/*----------------------------------------------------------------------------*/
typedef struct SrcWrapper_
{
    struct libsrc  super;
    uint32_t       processing_frame_num;
    int            src_max_output_frame_num; /* maximum frame num which SW_convert will output. */
} SrcWrapper;

/*----------------------------------------------------------------------------*/
static int
SW_initialize(struct libsrc* ls,
              uint32_t       input_sample_rate,
              uint32_t       output_sample_rate,
              uint32_t       processing_frame_num)
{
    SrcWrapper* sw = (SrcWrapper*)ls;

    if (input_sample_rate != 44100)
    {
        ALOGE("error input_sample_rate %d not supported", input_sample_rate);
        return -1;
    }

    if (output_sample_rate != 44100)
    {
        ALOGE("error output_sample_rate %d not supported", output_sample_rate);
        return -1;
    }

    sw->processing_frame_num     = processing_frame_num;
    sw->src_max_output_frame_num = sw->processing_frame_num;

    return 0;
}

static void
SW_finalize(struct libsrc* ls)
{
    /* do nothing */
}

static int
SW_get_max_output_frame_num(struct libsrc* ls)
{
    SrcWrapper* sw = (SrcWrapper*)ls;
    return sw->src_max_output_frame_num;
}

static int
SW_convert(struct libsrc*  ls,
           const uint64_t* input_buf,
           uint64_t*       output_buf,
           int*            output_frame_num)
{
    SrcWrapper* sw = (SrcWrapper*)ls;

    /* pass through */
    memcpy(output_buf, input_buf, sw->processing_frame_num * SIZE_OF_32BIT_PCM_FRAME);
    *output_frame_num = sw->processing_frame_num;

    return 0;
}

/*----------------------------------------------------------------------------*/
static int
get_libsrc(struct libsrc** pls)
{
    SrcWrapper* sw = (SrcWrapper*)malloc(sizeof(SrcWrapper));
    if (sw == NULL)
    {
        ALOGE("malloc failed");
        return -1;
    }

    sw->super.initialize               = SW_initialize;
    sw->super.finalize                 = SW_finalize;
    sw->super.get_max_output_frame_num = SW_get_max_output_frame_num;
    sw->super.convert                  = SW_convert;

    *pls = &sw->super;
    return 0;
}

static void
release_libsrc(struct libsrc* ls)
{
    SrcWrapper* sw = (SrcWrapper*)malloc(sizeof(SrcWrapper));
    free(sw);
}

/*----------------------------------------------------------------------------*/
struct libsrc_handle HAL_LIB_SRC = {
    .get_libsrc     = get_libsrc,
    .release_libsrc = release_libsrc
};


